import { ComponentMessage, HostMessage } from '../types';

export type MessageCallback = (message: ComponentMessage) => void;

export class MessageEventManager {
  private listeners: Map<string, Set<MessageCallback>> = new Map();
  private port?: MessagePort;

  constructor() {
    this.handleMessage = this.handleMessage.bind(this);
  }

  setPort(port: MessagePort): void {
    this.port = port;
    this.port.onmessage = this.handleMessage;
    this.port.start();
  }

  on(messageType: string, callback: MessageCallback): void {
    if (!this.listeners.has(messageType)) {
      this.listeners.set(messageType, new Set());
    }
    this.listeners.get(messageType)!.add(callback);
  }

  off(messageType: string, callback: MessageCallback): void {
    const callbacks = this.listeners.get(messageType);
    if (callbacks) {
      callbacks.delete(callback);
      if (callbacks.size === 0) {
        this.listeners.delete(messageType);
      }
    }
  }

  private handleMessage(event: MessageEvent): void {
    // console.log('received message', event.data);
    const message = event.data;
    if (typeof message === 'object' && message !== null && 'type' in message) {
      const messageType = message.type as string;
      const callbacks = this.listeners.get(messageType);
      if (callbacks) {
        callbacks.forEach((callback) => callback(message));
      }
    }
  }

  // Method to send messages through the port
  send(message: HostMessage): void {
    if (this.port) {
      this.port.postMessage(message);
    }
  }
}
